@@ -4,7 +4,7 @@ endorse <- function(Y,
                     village = NA,
                     treat = NA,
                     na.strings = 99,
-                    identical.lambda = FALSE,
+                    identical.lambda = TRUE,
                     covariates = FALSE,
                     formula.indiv = NA,
                     hierarchical = FALSE,
@@ -34,16 +34,16 @@ endorse <- function(Y,
                     precision.kappa = 0.04,
                     precision.delta = 0.04,
                     precision.zeta = 0.04,
-                    s0.omega2 = .2,
-                    nu0.omega2 = 30,
-                    s0.phi2 = .2,
-                    nu0.phi2 = 30,
-                    s0.psi2 = .2,
-                    nu0.psi2 = 30,
+                    s0.omega2 = 1,
+                    nu0.omega2 = 10,
+                    s0.phi2 = 1,
+                    nu0.phi2 = 10,
+                    s0.psi2 = 1,
+                    nu0.psi2 = 10,
                     s0.sig2 = 1,
                     nu0.sig2 = 400,
-                    s0.rho2 = .2,
-                    nu0.rho2 = 30,
+                    s0.rho2 = 1,
+                    nu0.rho2 = 10,
                     MCMC = 20000,
                     burn = 1000,
                     thin = 1,
